#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <avr/pgmspace.h>

#include "I2CSlave.h"

#define DURATION_LONG		700
#define DURATION_SHORT		150
#define BUZZER_PAUSE		50

#define CMD_BUZZER_ON				0x03U
#define CMD_BUZZER_OFF				0x04U

#define CMD_BUZZER_AUTH_SUCCESS		0x05U
#define CMD_BUZZER_AUTH_FAILED		0x06U

/* Red */
#define CMD_PD7_ON					0x15U
#define CMD_PD7_OFF					0x16U
/* Green */
#define CMD_PD6_ON					0x17U
#define CMD_PD6_OFF					0x18U
/* Blue */
#define CMD_PD5_ON					0x19U
#define CMD_PD5_OFF					0x1AU

uint8_t ATMEGA324_ADDR = 0x22U;
static volatile uint8_t _CMD = 0;

void IO_init(void)
{
	/* User LED */
	DDRD |= (1 << PD7);
	PORTD = 0;

	/* User Button */
	DDRB &= ~(1 << PB2);
	PORTB |= (1 << PB2);

	/* PB7 - Buzzer */
	DDRB |= (1 << PB7);
	PORTB &= ~(1 << PB7);

	/* PD6 - Green */
	DDRD |= (1 << PD6);
	PORTD &= ~(1 << PD6);

	/* PD5 - Blue */
	DDRD |= (1 << PD5);
	PORTD &= ~(1 << PD5);
}

void BUZZER_MakeNoise(int duration) {
	int i;

	for (i = 0; i < duration; ++i) {
		PINB = (1 << PB7);
		_delay_ms(1);
	}
}

void I2C_ReceiveCallback(uint8_t byte)
{
	_CMD = byte;
	// receiveBuffer[ind++] = byte;
	// ind %= RECV_BUFFER_SIZE;
	/* Echo the received byte */
	// I2C_transmitByte(byte);
}

void I2C_RequestCallback(void)
{
	I2C_transmitByte(_CMD);
	// I2C_transmitByte(receiveBuffer[ind++]);
	// ind %= RECV_BUFFER_SIZE;
}

void ATMEGA_CMD_LED(int pin, int sub_cmd)
{
	if (sub_cmd == 0)
		PORTD &= ~(1 << pin);
	else if (sub_cmd == 1)
		PORTD |= (1 << pin);
}

void ATMEGA_CMD_BUZZER(int sub_cmd)
{
	if (sub_cmd == 0)
	{
		ATMEGA_CMD_LED(PD6, 1);
		for (int i = 0; i < 3; ++i) {
			BUZZER_MakeNoise(DURATION_SHORT);
			PORTB &= ~(1 << PB7);
			_delay_ms(BUZZER_PAUSE);
		}
		BUZZER_MakeNoise(2 * DURATION_SHORT);
		PORTB &= ~(1 << PB7);
		ATMEGA_CMD_LED(PD6, 0);
	}
	else if (sub_cmd == 1)
	{
		PORTD |= (1 << PD7);
		ATMEGA_CMD_LED(PD7, 1);
		BUZZER_MakeNoise(DURATION_LONG);
		PORTB &= ~(1 << PB7);
		ATMEGA_CMD_LED(PD7, 0);
	}
	else if (sub_cmd == 2)
	{
		PORTB |= (1 << PB7);
		do {
			BUZZER_MakeNoise(DURATION_SHORT);
		} while (_CMD == CMD_BUZZER_ON);
	}
	else
	{
		PORTB &= ~(1 << PB7);
	}

}

int main (void)
{
	/* Init IO and I2C */
	IO_init();
	I2C_init(ATMEGA324_ADDR);
	I2C_setCallbacks(I2C_ReceiveCallback, I2C_RequestCallback);

	for(;;)
	{
		switch(_CMD)
		{
			case CMD_BUZZER_OFF:
				ATMEGA_CMD_BUZZER(3);
				break;
			case CMD_BUZZER_ON:
				ATMEGA_CMD_BUZZER(2);
				break;
			case CMD_BUZZER_AUTH_SUCCESS:
				ATMEGA_CMD_BUZZER(0);
				break;
			case CMD_BUZZER_AUTH_FAILED:
				ATMEGA_CMD_BUZZER(1);
				break;
			case CMD_PD7_OFF:
				ATMEGA_CMD_LED(PD7, 0);
				break;
			case CMD_PD7_ON:
				ATMEGA_CMD_LED(PD7, 1);
				break;
			case CMD_PD6_OFF:
				ATMEGA_CMD_LED(PD6, 0);
				break;
			case CMD_PD6_ON:
				ATMEGA_CMD_LED(PD6, 1);
				break;
			case CMD_PD5_OFF:
				ATMEGA_CMD_LED(PD5, 0);
				break;
			case CMD_PD5_ON:
				ATMEGA_CMD_LED(PD5, 1);
				break;
			default:
				break;
		}
	}

	return 0;
}
