#include "WiFi.h"
#include "ESPAsyncWebServer.h"
#include <Wire.h>
#include "BlueDot_BME280.h"
#include <SPI.h>
#include <MFRC522.h>

#define CMD_BUZZER_ON					    0x03U
#define CMD_BUZZER_OFF					  0x04U

#define CMD_BUZZER_AUTH_SUCCESS		0x05U
#define CMD_BUZZER_AUTH_FAILED		0x06U

/* Red */
#define CMD_PD7_ON					0x15U
#define CMD_PD7_OFF					0x16U
/* Green */
#define CMD_PD6_ON					0x17U
#define CMD_PD6_OFF					0x18U
/* Blue */
#define CMD_PD5_ON					0x19U
#define CMD_PD5_OFF					0x1AU

void SendCommand(uint8_t cmd);

/***************************************************************************************/
/******************************** Wifi module ******************************************/
/***************************************************************************************/

/* Networks cridentials */
const char* ssid     = "Bogdanovici";
const char* password = "pmfair2019";

/* Create AsyncWebServer object on port 80 */
AsyncWebServer server(80);

/* Define the main HTML pages and put them in program memory */
const char homepage_html[] PROGMEM = R"rawliteral(
<html>
<head>
    <title>EWS</title>
</head>
<body class = "body_bg">
<div <div align="center">
    <div <div align="center" class="topnav">
      <a class="active" href="/">HomePage</a>
      <a href="login">Authentication</a>
      <a id="met" href="meteo">%MET%</a>
      <a id="atmega" href="atmega">%ATMEGA%</a>
    </div>
    <h2>Wellcome to the Embedded Web Server!</h2>
</div>
</body>
</html>
<script>
setInterval(function ( ) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("met").innerHTML = this.responseText;
    }
  };
  xhttp.open("GET", "/homepage_met", true);
  xhttp.send();
}, 100 ) ;
setInterval(function ( ) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("atmega").innerHTML = this.responseText;
    }
  };
  xhttp.open("GET", "/homepage_atmega", true);
  xhttp.send();
}, 100 ) ;
</script>
<style>
  html {
    font-family: Arial;
    display: inline-block;
    margin: 0px auto;
    text-align: center;
  }
  h2 { font-size: 3.0rem; }
.topnav {
  background-color: #333;
  overflow: hidden;
}
.body_bg {
  background-color:#FFF2d3;
}
/* Style the links inside the navigation bar */
.topnav a {
  float: left;
  color: #f2f2f2;
  text-align: center;
  padding: 14px 16px;
  text-decoration: none;
  font-size: 17px;
}
/* Change the color of links on hover */
.topnav a:hover {
  background-color: #ddd;
  color: black;
}
/* Add a color to the active/current link */
.topnav a.active {
  background-color: #4CAF50;
  color: white;
}
</style>
)rawliteral";

const char login_html[] PROGMEM = R"rawliteral(
<html>
<head>
    <title>Card Authentication</title>
</head>
<body id="body_bg">
    <div <div align="center">
        <h3 id="msg">%MESSAGE%</h3>
    </div>
</body>
</html>
<script>
setInterval(function ( ) {
  var str1 = document.getElementById("msg").innerHTML;
  var str2 = "Authentication succeded!";
  var n = str1.localeCompare(str2);
  if (str1.localeCompare("Authentication succeded!") == 0) {
  	alert('User Authenticated!'); location.href = '/';
  } else if (str1.localeCompare("Authentication failed!") == 0) {
    alert('Athentication Failed!');
    var tid = setInterval(function(){},1000);
  } else {
    var xhttp = new XMLHttpRequest();
    xhttp.onreadystatechange = function() {
      if (this.readyState == 4 && this.status == 200) {
        document.getElementById("msg").innerHTML = this.responseText;
      }
    };
    xhttp.open("GET", "/msg", true);
    xhttp.send();
  }
}, 50 ) ;
</script>
<style>
    #body_bg {
        background-color: #FFF2d3;
    }
    .topnav {
        background-color: #333;
        overflow: hidden;
    }
    /* Style the links inside the navigation bar */
    .topnav a {
        float: left;
        color: #f2f2f2;
        text-align: center;
        padding: 14px 16px;
        text-decoration: none;
        font-size: 17px;
    }
    /* Change the color of links on hover */
    .topnav a:hover {
        background-color: #ddd;
        color: black;
    }
    /* Add a color to the active/current link */
    .topnav a.active {
        background-color: #4CAF50;
        color: white;
    }
    .button {
        display: inline-block;
        padding: 10px 15px;
        font-size: 20px;
        cursor: pointer;
        text-align: center;
        text-decoration: none;
        outline: none;
        color: #000;
        background-color: #c7e2ba;
        border: none;
        border-radius: 15px;
        box-shadow: 0 6px #999;
    }
    .button:hover {
        background-color: #3e8e41
    }
    .button:active {
        background-color: #3e8e41;
        box-shadow: 0 5px #666;
        transform: translateY(4px);
    }
</style>
)rawliteral";

const char meteo_html[] PROGMEM = R"rawliteral(
<!DOCTYPE HTML><html>
<head>
  <title>Meteo Station</title>
  <meta name="viewport" content="width=device-width, initial-scale=1">
  <link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.7.2/css/all.css" integrity="sha384-fnmOCqbTlWIlj8LyTjo7mOUStjsKC4pOpQbqyi7RrhN7udi9RwhKkMHpvLbHG9Sr" crossorigin="anonymous">
  <style>
    html {
     font-family: Arial;
     display: inline-block;
     margin: 0px auto;
     text-align: center;
    }
    h2 { font-size: 3.0rem; }
    p { font-size: 3.0rem; }
    .units { font-size: 1.2rem; }
    .bme280-labels{
      font-size: 1.5rem;
      vertical-align:middle;
      padding-bottom: 15px;
    }
    .topnav {
        background-color: #333;
        overflow: hidden;
    }
    /* Style the links inside the navigation bar */
    .topnav a {
        float: left;
        color: #f2f2f2;
        text-align: center;
        padding: 14px 16px;
        text-decoration: none;
        font-size: 17px;
    }
    /* Change the color of links on hover */
    .topnav a:hover {
        background-color: #ddd;
        color: black;
    }
    /* Add a color to the active/current link */
    .topnav a.active {
        background-color: #4CAF50;
        color: white;
    }
  </style>
</head>
<body id="body_bg">
  <div <div align="center" class="topnav">
      <a class="active" href="/">HomePage</a>
      <a href="login">Authentication</a>
      <a id="met" href="meteo">%MET%</a>
      <a id="atmega" href="atmega">%ATMEGA%</a>
  </div>
  <h2>Medium parameters</h2>
  <p>
    <i class="fas fa-thermometer-half" style="color:#059e8a;"></i>
    <span class="bme280-labels">Temperature</span>
    <span id="temperature">%TEMPERATURE%</span>
    <sup class="units">&deg;C</sup>
  </p>
  <p>
    <i class="fas fa-temperature-high"></i>
    <span class="bme280-labels">Pressure</span>
    <span id="pressure">%PRESSURE%</span>
    <sup class="units">hPa</sup>
  </p>
  <p>
    <i class="fas fa-tint" style="color:#00add6;"></i>
    <span class="bme280-labels">Humidity</span>
    <span id="humidity">%HUMIDITY%</span>
    <sup class="units">%%</sup>
  </p>
  <p>
    <i class="fas fa-sort-amount-up" style="color:#2E2E2E;"></i>
    <span class="bme280-labels">Altitude</span>
    <span id="altitude">%ALTITUDE%</span>
    <sup class="units">m</sup>
  </p>
</body>
<script>
setInterval(function ( ) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("temperature").innerHTML = this.responseText;
    }
  };
  xhttp.open("GET", "/temperature", true);
  xhttp.send();
}, 1000 ) ;

setInterval(function ( ) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("pressure").innerHTML = this.responseText;
    }
  };
  xhttp.open("GET", "/pressure", true);
  xhttp.send();
}, 1000 ) ;

setInterval(function ( ) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("humidity").innerHTML = this.responseText;
    }
  };
  xhttp.open("GET", "/humidity", true);
  xhttp.send();
}, 1000 ) ;

setInterval(function ( ) {
  var xhttp = new XMLHttpRequest();
  xhttp.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("altitude").innerHTML = this.responseText;
    }
  };
  xhttp.open("GET", "/altitude", true);
  xhttp.send();
}, 1000 ) ;
</script>
</html>
<style>
#body_bg {
        background-color: #FFF2d3;
    }
</style>)rawliteral";

const char atmega_html[] PROGMEM = R"rawliteral(
<html>
<head>
  <title>ATMEGA</title>
</head>
<body id="body_bg">

  <div <div align="center" class="topnav">
      <a class="active" href="/">HomePage</a>
      <a href="login">Authentication</a>
      <a id="met" href="meteo">%MET%</a>
      <a id="atmega" href="atmega">%ATMEGA%</a>
  </div>
  <div <div align="center">
    <h2>ATMEGA</h2>
    <table border="0.5" >
        <tr>
            <button id=butRed class="button button1" onclick="updateRed()">OFF</button>
        </tr>
        <tr>
            <button id=butGreen class="button button2" onclick="updateGreen()">OFF</button>
        </tr>
        <tr>
            <button id=butBlue class="button button3" onclick="updateBlue()">OFF</button>
        </tr>
        <tr>
            <button id=butBuzzer class="button button4" onclick="updateBuzzer()">OFF</button>
        </tr>
    </table>
	</div>

</body>
</html>

<script>
function updateRed() {
  var xhttpR = new XMLHttpRequest();
  xhttpR.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("butRed").innerHTML = this.responseText;
    }
  };
  xhttpR.open("GET", "/red", true);
  xhttpR.send();
}
function updateGreen( ) {
  var xhttpR = new XMLHttpRequest();
  xhttpR.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("butGreen").innerHTML = this.responseText;
    }
  };
  xhttpR.open("GET", "/green", true);
  xhttpR.send();
}
function updateBlue( ) {
  var xhttpR = new XMLHttpRequest();
  xhttpR.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("butBlue").innerHTML = this.responseText;
    }
  };
  xhttpR.open("GET", "/blue", true);
  xhttpR.send();
}
function updateBuzzer( ) {
  var xhttpR = new XMLHttpRequest();
  xhttpR.onreadystatechange = function() {
    if (this.readyState == 4 && this.status == 200) {
      document.getElementById("butBuzzer").innerHTML = this.responseText;
    }
  };
  xhttpR.open("GET", "/buzzer", true);
  xhttpR.send();
}
</script>
<style>
  html {
     font-family: Arial;
     display: inline-block;
     margin: 0px auto;
     text-align: center;
    }
  h2 { font-size: 3.0rem; }
  #body_bg
  {
  background-color:#FFF2d3;
  }

  .topnav {
    background-color: #333;
    overflow: hidden;
  color:white;
  }

  /* Style the links inside the navigation bar */
  .topnav a {
    float: left;
    color: #f2f2f2;
    text-align: center;
    padding: 14px 16px;
    text-decoration: none;
    font-size: 17px;
  }

  /* Change the color of links on hover */
  .topnav a:hover {
    background-color: #ddd;
    color: black;
  }

  /* Add a color to the active/current link */
  .topnav a.active {
    background-color: #4CAF50;
    color: white;
  }

  .button {
  background-color: #4CAF50; /* Green */
  border: none;
  color: white;
  padding: 16px 32px;
  text-align: center;
  text-decoration: none;
  display: block;
  font-size: 16px;
  margin: 4px 2px;
  -webkit-transition-duration: 0.4s; /* Safari */
  transition-duration: 0.4s;
  cursor: pointer;
}
.button2:hover {
  background-color: white;
  color: black;
  border: 2px solid #4CAF50;
}
.button2 {
  background-color: #4CAF50;
  color: white;
}
.button2:visited {
  background-color: #4CAF50;
  color: white;
}

.button3:hover {
  background-color: white;
  color: black;
  border: 2px solid #008CBA;
}
.button3 {
  background-color: #008CBA;
  color: white;
}
.button3:visited {
  background-color: #008CBA;
  color: white;
}

.button1:hover {
  background-color: white;
  color: black;
  border: 2px solid #f44336;
}
.button1 {
  background-color: #f44336;
  color: white;
}
.button1:visited {
  background-color: #f44336;
  color: white;
}

.button4:hover {
  background-color: white;
  color: black;
  border: 2px solid #555555;
}
.button4 {
  background-color: #555555;
  color: white;
}
.button4:visited {
  background-color: #555555;
  color: white;
}

</style>
)rawliteral";


/***************************************************************************************/
/************************************ BME280 *******************************************/
/***************************************************************************************/

BlueDot_BME280 bme;
int bmeDetected = 0;

void BME280_Init(void)
{
  Serial.println(F("Initializing BME280 sensor"));

  /* Set it for I2C mode */
  bme.parameter.communication = 0;

  /* I2C address */
  bme.parameter.I2CAddress = 0x76;

  /* Sensor mode - default mode in which measures continually (less accurate) */
  bme.parameter.sensorMode = 0b11;

  /* Set the IIR (Infinite Impulse Response) filter that suppresses high frequency fluctuations */
  /* to maximum */
  bme.parameter.IIRfilter = 0b100;

  /* Set oversampling at maximum for each (the least noise) */
  bme.parameter.humidOversampling = 0b101;
  bme.parameter.tempOversampling = 0b101;
  bme.parameter.pressOversampling = 0b101;

  /* Set the reference values for each */
  bme.parameter.pressureSeaLevel = 1013.25;
  bme.parameter.tempOutsideCelsius = 22;
  bme.parameter.tempOutsideFahrenheit = 59;

  if (bme.init() != 0x60)
  {
    Serial.println(F("Ops! BME280 Sensor not found!"));
    bmeDetected = 0;
  }
  else
  {
    Serial.println(F("BME280 Sensor detected!"));
    bmeDetected = 1;
  }
}

String BME280_ReadTempC() {
  float Tc;

  if (bmeDetected)
    Tc = bme.readTempC();
  Serial.println(Tc);
  return String(Tc);
}
String BME280_ReadPressure() {
  float P;

  if (bmeDetected)
    P = bme.readPressure();
  Serial.println(P);
  return String(P);
}
String BME280_ReadHumidity() {
  float H;

  if (bmeDetected)
    H = bme.readHumidity();
  Serial.println(H);
  return String(H);
}
String BME280_ReadAltitudeMeter() {
  float AltM;

  if (bmeDetected)
    AltM = bme.readAltitudeMeter();
  Serial.println(AltM);
  return String(AltM);
}

/* Replaces placeholder with sensor values */
String BME280_processor(const String& var){
  if(var == "TEMPERATURE"){
    return BME280_ReadTempC();
  }
  else if(var == "PRESSURE"){
    return BME280_ReadPressure();
  }
  else if(var == "HUMIDITY"){
    return BME280_ReadHumidity();
  }
  else if(var == "ALTITUDE"){
    return BME280_ReadAltitudeMeter();
  }
  return String();
}

/***************************************************************************************/
/*********************************** MFRC522 *******************************************/
/***************************************************************************************/
#define SS_PIN 5
#define RST_PIN 4

MFRC522 mfrc522(SS_PIN, RST_PIN);

int authState = 0;
int authenticated = 0;

static const char authorized_uid[] = "A6 BF 2D C7";
static char current_user_uid[20];

int CheckAuthentication(void)
{
  String content= "";
  byte letter;

  Serial.print("UID tag :");
  for (byte i = 0; i < mfrc522.uid.size; i++)
  {
     Serial.print(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " ");
     Serial.print(mfrc522.uid.uidByte[i], HEX);
     content.concat(String(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " "));
     content.concat(String(mfrc522.uid.uidByte[i], HEX));
  }
  Serial.println();
  Serial.print("Message : ");
  content.toUpperCase();
  if (content.substring(1) == authorized_uid)
  {
    Serial.println("Authorized access");
    Serial.println();
    return 1;
  }
  else   {
      Serial.println(" Access denied");
      return 0;
    }
}

String Login_processor(const String& var){
  if(authState == 0) {
    if (mfrc522.PICC_IsNewCardPresent()) {
      authState = 1;
      return String("Card detected..");
    }
    else
      return String("Present your card...");
  }
  else if(authState == 1){
    delay(600);
    SendCommand(CMD_PD5_ON);
    if (mfrc522.PICC_ReadCardSerial()) {
      authState = 2;
      return String("Reading card.");
    }
    else if (!mfrc522.PICC_IsNewCardPresent()) {
      authState = 0;
      SendCommand(CMD_PD5_OFF);
      return String("Present your card...");
    }
    else
      return String("Card detected..");
  }
  else if(authState == 2){
    delay(600);
    SendCommand(CMD_PD5_OFF);
    if (CheckAuthentication()) {
      authState = 3;
      SendCommand(CMD_BUZZER_AUTH_SUCCESS);
      delay(100);
      SendCommand(0x00U);
      authenticated = 1;
      return String("Authentication succeded!");
    }
    else{
      authState = 4;
      SendCommand(CMD_BUZZER_AUTH_FAILED);
      delay(100);
      SendCommand(0x00U);
      authenticated = 0;
      return String("Authentication failed!");
    }
  }
  else {
    delay(1000);
    authState = 0;
  }

  return String();
}

/***************************************************************************************/
/********************************** ATMEGA324P *****************************************/
/***************************************************************************************/
uint8_t ATMEGA324_ADDR = 0x22U;

int PD7_state = 0;
int PD6_state = 0;
int PD5_state = 0;
int BUZZER_state = 0;

void SendCommand(uint8_t cmd)
{
  int timeout = 1000;

  Wire.beginTransmission(ATMEGA324_ADDR);
  Wire.write(cmd);
  Wire.endTransmission();

  Serial.println("Sent!");

  do {

    int r1 = 0xAB;
    Wire.requestFrom(ATMEGA324_ADDR, 1);

    if(Wire.available()<=1) {
      int av = Wire.available();
      Serial.print("Available: ");
      Serial.println(Wire.available());
      r1 = Wire.read();
      Serial.print("From ATMEGA: ");
      Serial.println(r1);
    }

    if (r1 == cmd) {
      Serial.println("Received confirmation!");
      break;
    }

    delay(1);
  } while(timeout-- > 0);
}

String ATMEGA_processor(const String& var){
  if (var == "RED_STATE") {
    if(PD7_state) {
      SendCommand(CMD_PD7_OFF);
      delay(10);
      SendCommand(0x00);
      PD7_state = 0;
      return String("OFF");
    }
    else {
      SendCommand(CMD_PD7_ON);
      delay(10);
      SendCommand(0x00);
      PD7_state = 1;
      return String("ON");
    }
  }
  else if (var == "GREEN_STATE") {
    if(PD6_state) {
      SendCommand(CMD_PD6_OFF);
      delay(10);
      SendCommand(0x00);
      PD6_state = 0;
      return String("OFF");
    }
    else {
      SendCommand(CMD_PD6_ON);
      delay(10);
      SendCommand(0x00);
      PD6_state = 1;
      return String("ON");
    }
  }
  else if (var == "BLUE_STATE") {
    if(PD5_state) {
      SendCommand(CMD_PD5_OFF);
      delay(10);
      SendCommand(0x00);
      PD5_state = 0;
      return String("OFF");
    }
    else {
      SendCommand(CMD_PD5_ON);
      delay(10);
      SendCommand(0x00);
      PD5_state = 1;
      return String("ON");
    }
  }
  else if (var == "BUZZER_STATE") {
    if(BUZZER_state) {
      SendCommand(CMD_BUZZER_OFF);
      delay(10);
      SendCommand(0x00);
      BUZZER_state = 0;
      return String("OFF");
    }
    else {
      SendCommand(CMD_BUZZER_ON);
      BUZZER_state = 1;
      return String("ON");
    }
  }
  return String();
}

String ATMEGA_state(const String& var){
  if (var == "RED_STATE") {
    if(PD7_state) {
      SendCommand(CMD_PD7_OFF);
      delay(100);
      SendCommand(0x00);
      PD7_state = 0;
      return String("OFF");
    }
    else {
      SendCommand(CMD_PD7_ON);
      delay(100);
      SendCommand(0x00);
      PD7_state = 1;
      return String("ON");
    }
  }
  else if (var == "GREEN_STATE") {
    if(PD6_state) {
      SendCommand(CMD_PD6_OFF);
      delay(100);
      SendCommand(0x00);
      PD6_state = 0;
      return String("OFF");
    }
    else {
      SendCommand(CMD_PD6_ON);
      delay(100);
      SendCommand(0x00);
      PD6_state = 1;
      return String("ON");
    }
  }
  else if (var == "BLUE_STATE") {
    if(PD5_state) {
      SendCommand(CMD_PD5_OFF);
      delay(100);
      SendCommand(0x00);
      PD5_state = 0;
      return String("OFF");
    }
    else {
      SendCommand(CMD_PD5_ON);
      delay(100);
      SendCommand(0x00);
      PD5_state = 1;
      return String("ON");
    }
  }
  else if (var == "BUZZER_STATE") {
    if(BUZZER_state) {
      SendCommand(CMD_BUZZER_OFF);
      delay(100);
      SendCommand(0x00);
      BUZZER_state = 0;
      return String("OFF");
    }
    else {
      SendCommand(CMD_BUZZER_ON);
      delay(100);
      SendCommand(0x00);
      BUZZER_state = 1;
      return String("ON");
    }
  }
  return String();
}

String HomePage_processor(const String& var){
  if(authenticated != 0) {
    if(var == "MET"){
      return String("Meteo Station");
    }
    else if (var == "ATMEGA") {
      return String("Playground");
    }
  }
  return String();
}

/***************************************************************************************/
/******************************** Setup/Loop *******************************************/
/***************************************************************************************/
void setup(){
  /* Serial port for debugging purposes */
  Serial.begin(115200);

  /* I2C init */
  Wire.begin();
  Wire.setClock(400000UL); /*400000UL*/
  delay(100);

  /* Init mfrc522 */
  SPI.begin();
  mfrc522.PCD_Init();

  /* Init seonsors (BME280) */
  BME280_Init();

  /* Connect to Wi-Fi */
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Connecting to WiFi..");
  }

  /* Print Local IP Address */
  Serial.println(WiFi.localIP());

  /* Web pages routing */
  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/html", homepage_html, HomePage_processor);
  });
  server.on("/login", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/html", login_html, Login_processor);
  });
  server.on("/meteo", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/html", meteo_html, BME280_processor);
  });
  server.on("/atmega", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/html", atmega_html, ATMEGA_state);
  });

  server.on("/temperature", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", BME280_ReadTempC().c_str());
  });
  server.on("/pressure", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", BME280_ReadPressure().c_str());
  });
  server.on("/humidity", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", BME280_ReadHumidity().c_str());
  });
  server.on("/altitude", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", BME280_ReadAltitudeMeter().c_str());
  });
  server.on("/msg", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", Login_processor("").c_str());
  });

  server.on("/homepage_met", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", HomePage_processor("MET").c_str());
  });
  server.on("/homepage_atmega", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", HomePage_processor("ATMEGA").c_str());
  });

  server.on("/red", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", ATMEGA_processor("RED_STATE").c_str());
  });
  server.on("/green", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", ATMEGA_processor("GREEN_STATE").c_str());
  });
  server.on("/blue", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", ATMEGA_processor("BLUE_STATE").c_str());
  });
  server.on("/buzzer", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/plain", ATMEGA_processor("BUZZER_STATE").c_str());
  });

  /* Start server */
  server.begin();
}

void loop()
{

}
